<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Util\Functions;

class RelFechCaixaControle{
    
    protected $relatorioService;
    protected $utilService;
    protected $birtReport;
    protected $dataSourceName = 'vnd05606_relFechamentoCaixaControle';
    protected $nameReport = 'fechCaixa';
    protected $retirarParams = array('NMLOJA','NMFILIAL','changed','DTENTRVENDA','CDLOJACAIXA','NMCAIXA');
    protected $inserirParams = array(
        'IDSITUVENDA'=>'O',
        'IDTIPOMOVIVEE'=>'E',
        'IDCONFVENDA1'=>'1',
        'IDTIPORECE7'=>'7',
        'IDTIPOMOVIVES'=>'S',
        'IDTIPOMOVIVEG'=>'G',
        'IDTIPOMOVIVET'=>'T'
    );
    
    
    public function __construct (Utilities\Util $utilService,ReportService $reportService, Services\Relatorio $relatorioService, BirtReport $birtReport) {
        $this->reportService = $reportService;
        $this->relatorioService = $relatorioService;
        $this->utilService = $utilService;
        $this->birtReport = $birtReport;
    }
    
    private function getNameReport($name){
        if($name==''){
            return $this->nameReport.'Control';
        }
        return $this->nameReport.$name;
    }
    
     public function geraRelatorio(DTO\Request\Row $request, DTO\Response $response) {
         $conditions = $this->utilService->getParamsAsArray($request);
         $conditions = $this->buildParams('CDFILIAL',$conditions);
         $conditions = $this->buildParams('CDLOJA',$conditions);
         if(isset($conditions['CDCAIXA'])){
             $conditions['CDCAIXA'] = $this->buildParamsCaixa($conditions['CDCAIXA']);
         }
         $conditions = $this->buildParams('CDCAIXA',$conditions);
         $conditions = $this->buildDataVenda($conditions);
         $conditions = $this->retirarParams($conditions);
         $conditions = $this->inserirParams($conditions);
         $conditions['__report_name'] = $this->getNameReport(isset($conditions['TIPORELATORIO'])?$conditions['TIPORELATORIO']:"");
         $params = $this->buildParamsArray($conditions);
         $params = array(
            'ParametersRow' => $params,
            'ReportName'    => $conditions['__report_name']
        );
         $this->getRelatorio($params, $response);
     }
     
     public function getRelatorio ($relatorioData , DTO\Response $response){
         try {
            $birtURL = $this->reportService->createRemoteReport('BIRT', $relatorioData['ReportName'], $relatorioData['ParametersRow']);
            $response->addDataset(new DataSet('URL',array('url'=>$birtURL)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage() , 500));
        }
     }
     
     private function buildParams ($key, $conditions){
         if(!Functions::arrayKeyExists($key, $conditions) || !count($conditions[$key])) {
            $conditions[$key] = '';
            $conditions['T'.$key] = 'T';
        }else{
            $conditions[$key] = implode('_', $conditions[$key]);
            $conditions['T'.$key] = '';
        }
        return $conditions;
     }
     
    private function buildDataVenda ($conditions){
        if(Functions::arrayKeyExists('DTENTRVENDA', $conditions)) {
            $conditions['DATAINI'] = $conditions['DTENTRVENDA']['start'];
            $conditions['DATAFIN'] = $conditions['DTENTRVENDA']['end'];
        }
        return $conditions;
    }
    
    private function retirarParams($conditions){
        foreach ($this->retirarParams as $value) {
            if(isset($conditions[$value])){
                unset($conditions[$value]);
            }
        }
        return $conditions;
    }
    
    private function inserirParams($conditions){
        foreach ($this->inserirParams as $key =>  $value) {
            $conditions[$key] = $value;
        }
        return $conditions;
    }
    
    private function parameterMapping($conditions){
        $parameterMapping = array();
        foreach ($conditions as $key=>$value) {
            $parameterMapping[$key] = $key;
        }
        unset($parameterMapping['__report_name']);
        return $parameterMapping;
    }
    
    private function buildParamsCaixa ($param){
        return str_replace('_','',$param);
    }
    
    private function buildParamsArray($conditions){
        $params = array(
            "TIPORELATORIO" => $conditions['TIPORELATORIO'],
            "LISTAR_PRODUTO"=> $conditions['LISTAR_PRODUTO'],
            "CDFILIAL"      => $conditions['CDFILIAL'],
            "CDLOJA"        => $conditions['CDLOJA'],
            "CDCAIXA"       => $conditions['CDCAIXA'],
            "NRORG"         => $conditions['NRORG'],
            "CDOPERADOR"    => $conditions['CDOPERADOR'],
            "TCDFILIAL"     => $conditions['TCDFILIAL'],
            "TCDLOJA"       => $conditions['TCDLOJA'],
            "TCDCAIXA"      => $conditions['TCDCAIXA'],
            "DATAINI"       => $conditions['DATAINI'],
            "DATAFIN"       => $conditions['DATAFIN'],
            "IDSITUVENDA"   => $conditions['IDSITUVENDA'],
            "IDTIPOMOVIVEE" => $conditions['IDTIPOMOVIVEE'],
            "IDCONFVENDA1"  => $conditions['IDCONFVENDA1'],
            "IDTIPORECE7"   => $conditions['IDTIPORECE7'],
            "IDTIPOMOVIVES" => $conditions['IDTIPOMOVIVES'],
            "IDTIPOMOVIVEG" => $conditions['IDTIPOMOVIVEG'],
            "IDTIPOMOVIVET" => $conditions['IDTIPOMOVIVET']
        );
        return $params;
    }
}
